@@ -18,7 +18,7 @@ import pytest
 import coverage
 from coverage import env
 from coverage.data import line_counts
-from coverage.exceptions import CoverageException
+from coverage.exceptions import CoverageException, DataError, NoDataError, NoSource
 from coverage.files import abs_file, relative_filename
 from coverage.misc import import_local_file
 
@@ -26,6 +26,7 @@ from tests.coveragetest import CoverageTest, TESTS_DIR, UsingModulesMixin
 from tests.helpers import assert_count_equal, assert_coverage_warnings
 from tests.helpers import change_dir, nice_file, os_sep
 
+BAD_SQLITE_REGEX = r"file( is encrypted or)? is not a database"
 
 class ApiTest(CoverageTest):
     """Api-oriented tests for coverage.py."""
@@ -293,7 +294,7 @@ class ApiTest(CoverageTest):
         # empty summary reports raise exception, just like the xml report
         cov = coverage.Coverage()
         cov.erase()
-        with pytest.raises(CoverageException, match="No data to report."):
+        with pytest.raises(NoDataError, match="No data to report."):
             cov.report()
 
     def test_completely_zero_reporting(self):
@@ -322,7 +323,7 @@ class ApiTest(CoverageTest):
         )
         self.make_file(".coverage", cov4_data)
         cov = coverage.Coverage()
-        with pytest.raises(CoverageException, match="Looks like a coverage 4.x data file"):
+        with pytest.raises(DataError, match="Looks like a coverage 4.x data file"):
             cov.load()
         cov.erase()
 
@@ -422,7 +423,7 @@ class ApiTest(CoverageTest):
         self.make_file(".coverage.foo", """La la la, this isn't coverage data!""")
         cov = coverage.Coverage()
         warning_regex = (
-            r"Couldn't use data file '.*\.coverage\.foo': file (is encrypted or )?is not a database"
+            r"Couldn't use data file '.*\.coverage\.foo': " + BAD_SQLITE_REGEX
         )
         with self.assert_warnings(cov, [warning_regex]):
             cov.combine()
@@ -445,7 +446,7 @@ class ApiTest(CoverageTest):
         self.assert_exists(".coverage")
 
         cov2 = coverage.Coverage()
-        with pytest.raises(CoverageException, match=r"No data to combine"):
+        with pytest.raises(NoDataError, match=r"No data to combine"):
             cov2.combine(strict=True, keep=False)
 
         cov3 = coverage.Coverage()
@@ -479,10 +480,13 @@ class ApiTest(CoverageTest):
         # https://github.com/nedbat/coveragepy/issues/649
         # The order of the [paths] setting matters
         def make_data_file():
-            data = coverage.CoverageData(".coverage.1")
-            data.add_lines({abs_file('ci/girder/g1.py'): dict.fromkeys(range(10))})
-            data.add_lines({abs_file('ci/girder/plugins/p1.py'): dict.fromkeys(range(10))})
-            data.write()
+            self.make_data_file(
+                basename=".coverage.1",
+                lines={
+                    abs_file('ci/girder/g1.py'): range(10),
+                    abs_file('ci/girder/plugins/p1.py'): range(10),
+                },
+            )
 
         def get_combined_filenames():
             cov = coverage.Coverage()
@@ -854,6 +858,17 @@ class IncludeOmitTestsMixin(UsingModulesMixin, CoverageTest):
 class SourceIncludeOmitTest(IncludeOmitTestsMixin, CoverageTest):
     """Test using `source`, `include`, and `omit` when measuring code."""
 
+    def setUp(self):
+        super().setUp()
+
+        # These tests use the TESTS_DIR/modules files, but they cd into it. To
+        # keep tests from cross-contaminating, we make a copy of the files.
+        # Since we need to import from there, we also add it to the beginning
+        # of sys.path.
+
+        shutil.copytree(nice_file(TESTS_DIR, "modules"), "tests_dir_modules")
+        sys.path.insert(0, abs_file("tests_dir_modules"))
+
     def coverage_usepkgs(self, **kwargs):
         """Run coverage on usepkgs and return the line summary.
 
@@ -864,7 +879,8 @@ class SourceIncludeOmitTest(IncludeOmitTestsMixin, CoverageTest):
         cov.start()
         import usepkgs  # pragma: nested   # pylint: disable=import-error, unused-import
         cov.stop()      # pragma: nested
-        data = cov.get_data()
+        with self.assert_warnings(cov, []):
+            data = cov.get_data()
         summary = line_counts(data)
         for k, v in list(summary.items()):
             assert k.endswith(".py")
@@ -886,7 +902,7 @@ class SourceIncludeOmitTest(IncludeOmitTestsMixin, CoverageTest):
         assert lines['p1c'] == 0
 
     def test_source_package_as_dir(self):
-        os.chdir(nice_file(TESTS_DIR, "modules"))
+        os.chdir("tests_dir_modules")
         assert os.path.isdir("pkg1")
         lines = self.coverage_usepkgs(source=["pkg1"])
         self.filenames_in(lines, "p1a p1b")
@@ -912,7 +928,7 @@ class SourceIncludeOmitTest(IncludeOmitTestsMixin, CoverageTest):
         # the search for unexecuted files, and given a score of 0%.
 
         # The omit arg is by path, so need to be in the modules directory.
-        os.chdir(nice_file(TESTS_DIR, "modules"))
+        os.chdir("tests_dir_modules")
         lines = self.coverage_usepkgs(source=["pkg1"], omit=["pkg1/p1b.py"])
         self.filenames_in(lines, "p1a")
         self.filenames_not_in(lines, "p1b")
@@ -926,16 +942,16 @@ class SourceIncludeOmitTest(IncludeOmitTestsMixin, CoverageTest):
         assert lines['p1c'] == 0
 
     def test_ambiguous_source_package_as_dir(self):
-        # pkg1 is a directory and a pkg, since we cd into tests/modules/ambiguous
-        os.chdir(nice_file(TESTS_DIR, "modules", "ambiguous"))
-        # pkg1 defaults to directory because tests/modules/ambiguous/pkg1 exists
+        # pkg1 is a directory and a pkg, since we cd into tests_dir_modules/ambiguous
+        os.chdir("tests_dir_modules/ambiguous")
+        # pkg1 defaults to directory because tests_dir_modules/ambiguous/pkg1 exists
         lines = self.coverage_usepkgs(source=["pkg1"])
         self.filenames_in(lines, "ambiguous")
         self.filenames_not_in(lines, "p1a p1b p1c")
 
     def test_ambiguous_source_package_as_package(self):
-        # pkg1 is a directory and a pkg, since we cd into tests/modules/ambiguous
-        os.chdir(nice_file(TESTS_DIR, "modules", "ambiguous"))
+        # pkg1 is a directory and a pkg, since we cd into tests_dir_modules/ambiguous
+        os.chdir("tests_dir_modules/ambiguous")
         lines = self.coverage_usepkgs(source_pkgs=["pkg1"])
         self.filenames_in(lines, "p1a p1b")
         self.filenames_not_in(lines, "p2a p2b othera otherb osa osb ambiguous")
@@ -1126,7 +1142,7 @@ class RelativePathTest(CoverageTest):
         with change_dir("new"):
             cov = coverage.Coverage()
             cov.load()
-            with pytest.raises(CoverageException, match=expected):
+            with pytest.raises(NoSource, match=expected):
                 cov.report()
 
     def test_moving_stuff_with_relative(self):
@@ -1199,3 +1215,209 @@ class RelativePathTest(CoverageTest):
         assert files == {'foo.py', 'bar.py', os_sep('modsrc/__init__.py')}
         res = cov.report()
         assert res == 100
+
+    def test_combine_no_suffix_multiprocessing(self):
+        self.make_file(".coveragerc", """\
+            [run]
+            branch = True
+            """)
+        cov = coverage.Coverage(
+            config_file=".coveragerc",
+            concurrency="multiprocessing",
+            data_suffix=False,
+            )
+        cov.start()
+        cov.stop()
+        # The warning isn't the point of this test, but suppress it.
+        with pytest.warns(Warning) as warns:
+            cov.combine()
+        assert_coverage_warnings(warns, "No data was collected. (no-data-collected)")
+        cov.save()
+        self.assert_file_count(".coverage.*", 0)
+        self.assert_exists(".coverage")
+
+
+class CombiningTest(CoverageTest):
+    """More tests of combining data."""
+
+    B_LINES = {"b_or_c.py": [1, 2, 3, 4, 8, 9]}
+    C_LINES = {"b_or_c.py": [1, 2, 3, 6, 7, 8, 9]}
+
+    def make_b_or_c_py(self):
+        """Create b_or_c.py, used in a few of these tests."""
+        # "b_or_c.py b" will run 6 lines.
+        # "b_or_c.py c" will run 7 lines.
+        # Together, they run 8 lines.
+        self.make_file("b_or_c.py", """\
+            import sys
+            a = 2
+            if sys.argv[1] == 'b':
+                b = 4
+            else:
+                c = 6
+                c2 = 7
+            d = 8
+            print('done')
+            """)
+
+    def test_combine_parallel_data(self):
+        self.make_b_or_c_py()
+        self.make_data_file(".coverage.b", lines=self.B_LINES)
+        self.make_data_file(".coverage.c", lines=self.C_LINES)
+
+        # Combine the parallel coverage data files into .coverage .
+        cov = coverage.Coverage()
+        cov.combine(strict=True)
+        self.assert_exists(".coverage")
+
+        # After combining, there should be only the .coverage file.
+        self.assert_file_count(".coverage.*", 0)
+
+        # Read the coverage file and see that b_or_c.py has all 8 lines
+        # executed.
+        data = coverage.CoverageData()
+        data.read()
+        assert line_counts(data)['b_or_c.py'] == 8
+
+        # Running combine again should fail, because there are no parallel data
+        # files to combine.
+        cov = coverage.Coverage()
+        with pytest.raises(NoDataError, match=r"No data to combine"):
+            cov.combine(strict=True)
+
+        # And the originally combined data is still there.
+        data = coverage.CoverageData()
+        data.read()
+        assert line_counts(data)['b_or_c.py'] == 8
+
+    def test_combine_parallel_data_with_a_corrupt_file(self):
+        self.make_b_or_c_py()
+        self.make_data_file(".coverage.b", lines=self.B_LINES)
+        self.make_data_file(".coverage.c", lines=self.C_LINES)
+
+        # Make a bogus data file.
+        self.make_file(".coverage.bad", "This isn't a coverage data file.")
+
+        # Combine the parallel coverage data files into .coverage .
+        cov = coverage.Coverage()
+        with pytest.warns(Warning) as warns:
+            cov.combine(strict=True)
+        assert_coverage_warnings(
+            warns,
+            re.compile(
+                r"Couldn't use data file '.*[/\\]\.coverage\.bad': " + BAD_SQLITE_REGEX
+            ),
+        )
+
+        # After combining, those two should be the only data files.
+        self.assert_exists(".coverage")
+        self.assert_exists(".coverage.bad")
+        self.assert_file_count(".coverage.*", 1)
+
+        # Read the coverage file and see that b_or_c.py has all 8 lines
+        # executed.
+        data = coverage.CoverageData()
+        data.read()
+        assert line_counts(data)['b_or_c.py'] == 8
+
+    def test_combine_no_usable_files(self):
+        # https://github.com/nedbat/coveragepy/issues/629
+        self.make_b_or_c_py()
+        self.make_data_file(".coverage", lines=self.B_LINES)
+
+        # Make bogus data files.
+        self.make_file(".coverage.bad1", "This isn't a coverage data file.")
+        self.make_file(".coverage.bad2", "This isn't a coverage data file.")
+
+        # Combine the parallel coverage data files into .coverage, but nothing is readable.
+        cov = coverage.Coverage()
+        with pytest.warns(Warning) as warns:
+            with pytest.raises(NoDataError, match=r"No usable data files"):
+                cov.combine(strict=True)
+
+        warn_rx = re.compile(
+            r"Couldn't use data file '.*[/\\]\.coverage\.bad[12]': " + BAD_SQLITE_REGEX
+        )
+        assert_coverage_warnings(warns, warn_rx, warn_rx)
+
+        # After combining, we should have a main file and two parallel files.
+        self.assert_exists(".coverage")
+        self.assert_exists(".coverage.bad1")
+        self.assert_exists(".coverage.bad2")
+        self.assert_file_count(".coverage.*", 2)
+
+        # Read the coverage file and see that b_or_c.py has 6 lines
+        # executed (we only did b, not c).
+        data = coverage.CoverageData()
+        data.read()
+        assert line_counts(data)['b_or_c.py'] == 6
+
+    def test_combine_parallel_data_in_two_steps(self):
+        self.make_b_or_c_py()
+        self.make_data_file(".coverage.b", lines=self.B_LINES)
+
+        # Combine the (one) parallel coverage data file into .coverage .
+        cov = coverage.Coverage()
+        cov.combine(strict=True)
+
+        self.assert_exists(".coverage")
+        self.assert_file_count(".coverage.*", 0)
+
+        self.make_data_file(".coverage.c", lines=self.C_LINES)
+        self.assert_exists(".coverage")
+        self.assert_file_count(".coverage.*", 1)
+
+        # Combine the parallel coverage data files into .coverage .
+        cov = coverage.Coverage()
+        cov.load()
+        cov.combine(strict=True)
+
+        # After combining, there should be only the .coverage file.
+        self.assert_exists(".coverage")
+        self.assert_file_count(".coverage.*", 0)
+
+        # Read the coverage file and see that b_or_c.py has all 8 lines
+        # executed.
+        data = coverage.CoverageData()
+        data.read()
+        assert line_counts(data)['b_or_c.py'] == 8
+
+    def test_combine_parallel_data_no_append(self):
+        self.make_b_or_c_py()
+        self.make_data_file(".coverage.b", lines=self.B_LINES)
+
+        # Combine the (one) parallel coverage data file into .coverage .
+        cov = coverage.Coverage()
+        cov.combine(strict=True)
+        self.assert_exists(".coverage")
+        self.assert_file_count(".coverage.*", 0)
+
+        self.make_data_file(".coverage.c", lines=self.C_LINES)
+
+        # Combine the parallel coverage data files into .coverage, but don't
+        # use the data in .coverage already.
+        cov = coverage.Coverage()
+        cov.combine(strict=True)
+
+        # After combining, there should be only the .coverage file.
+        self.assert_exists(".coverage")
+        self.assert_file_count(".coverage.*", 0)
+
+        # Read the coverage file and see that b_or_c.py has only 7 lines
+        # because we didn't keep the data from running b.
+        data = coverage.CoverageData()
+        data.read()
+        assert line_counts(data)['b_or_c.py'] == 7
+
+    def test_combine_parallel_data_keep(self):
+        self.make_b_or_c_py()
+        self.make_data_file(".coverage.b", lines=self.B_LINES)
+        self.make_data_file(".coverage.c", lines=self.C_LINES)
+
+        # Combine the parallel coverage data files into .coverage with the keep flag.
+        cov = coverage.Coverage()
+        cov.combine(strict=True, keep=True)
+
+        # After combining, the .coverage file & the original combined file should still be there.
+        self.assert_exists(".coverage")
+        self.assert_file_count(".coverage.*", 2)
