@@ -1515,18 +1515,14 @@ class OptimizedIfTest(CoverageTest):
         )
 
     def test_if_not_debug(self):
-        arcz_missing = ""
-        if env.PYBEHAVIOR.pep626:
+        if env.PYBEHAVIOR.optimize_if_not_debug == 1:
             arcz = ".1 12 23 34 42 37 72 28 8."
-        elif env.PYBEHAVIOR.optimize_if_not_debug3:
-            arcz = ".1 12 23 32 37 72 28 8."
-        elif env.PYBEHAVIOR.optimize_if_not_debug2:
+        elif env.PYBEHAVIOR.optimize_if_not_debug == 2:
             arcz = ".1 12 23 35 52 37 72 28 8."
-        elif env.PYBEHAVIOR.optimize_if_not_debug:
-            arcz = ".1 12 23 34 42 37 72 28 8."
         else:
-            arcz = ".1 12 23 34 45 42 52 37 72 28 8."
-            arcz_missing = "45 52"
+            assert env.PYBEHAVIOR.optimize_if_not_debug == 3
+            arcz = ".1 12 23 32 37 72 28 8."
+
         self.check_coverage("""\
             lines = set()
             for value in [True, False]:
@@ -1538,7 +1534,6 @@ class OptimizedIfTest(CoverageTest):
             assert lines == set([7])
             """,
             arcz=arcz,
-            arcz_missing=arcz_missing,
         )
 
 
@@ -1716,6 +1711,7 @@ class DecoratorArcTest(CoverageTest):
             arcz = arcz.replace("3.", "34 4.")
         if env.PYBEHAVIOR.trace_decorator_line_again:
             arcz += "43 "
+        # This example makes more sense when considered in tandem with 466b below.
         self.check_coverage("""\
             class Parser(object):
 
@@ -1832,7 +1828,7 @@ class LambdaArcTest(CoverageTest):
 
 
 xfail_eventlet_670 = pytest.mark.xfail(
-    env.PYVERSION[:2] == (3, 9) and env.OSX,
+    env.PYVERSION[:2] == (3, 9) and env.CPYTHON and env.OSX,
     reason="Avoid an eventlet bug on Mac 3.9: eventlet#670",
     # https://github.com/eventlet/eventlet/issues/670
 )
