@@ -3,6 +3,7 @@
 
 """Test cmdline.py for coverage.py."""
 
+import ast
 import pprint
 import re
 import sys
@@ -20,7 +21,7 @@ from coverage.exceptions import _ExceptionDuringRun
 from coverage.version import __url__
 
 from tests.coveragetest import CoverageTest, OK, ERR, command_line
-from tests.helpers import os_sep
+from tests.helpers import os_sep, re_line
 
 
 class BaseCmdLineTest(CoverageTest):
@@ -272,7 +273,7 @@ class CmdLineTest(BaseCmdLineTest):
             """)
 
     @pytest.mark.parametrize("cmd, output", [
-        ("debug", "What information would you like: config, data, sys, premain?"),
+        ("debug", "What information would you like: config, data, sys, premain, pybehave?"),
         ("debug foo", "Don't know what you mean by 'foo'"),
         ("debug sys config", "Only one topic at a time, please"),
     ])
@@ -292,6 +293,16 @@ class CmdLineTest(BaseCmdLineTest):
         assert "skip_covered:" in out
         assert "skip_empty:" in out
 
+    def test_debug_pybehave(self):
+        self.command_line("debug pybehave")
+        out = self.stdout()
+        assert " CPYTHON:" in out
+        assert " PYVERSION:" in out
+        assert " pep626:" in out
+        pyversion = re_line(r" PYVERSION:", out)
+        vtuple = ast.literal_eval(pyversion.partition(":")[-1].strip())
+        assert vtuple[:5] == sys.version_info
+
     def test_debug_premain(self):
         self.command_line("debug premain")
         out = self.stdout()
@@ -1036,7 +1047,7 @@ class CmdMainTest(CoverageTest):
                 try:
                     raise Exception("oh noes!")
                 except:
-                    raise _ExceptionDuringRun(*sys.exc_info())
+                    raise _ExceptionDuringRun(*sys.exc_info()) from None
             elif argv[0] == 'internalraise':
                 raise ValueError("coverage is broken")
             elif argv[0] == 'exit':
