@@ -5,6 +5,7 @@
 
 import glob
 import io
+import math
 import os
 import os.path
 import py_compile
@@ -16,10 +17,12 @@ import coverage
 from coverage import env
 from coverage.control import Coverage
 from coverage.data import CoverageData
-from coverage.exceptions import CoverageException
+from coverage.exceptions import ConfigError, NoDataError, NotPython
+from coverage.files import abs_file
 from coverage.summary import SummaryReporter
 
 from tests.coveragetest import CoverageTest, TESTS_DIR, UsingModulesMixin
+from tests.helpers import assert_coverage_warnings
 
 
 class SummaryTest(UsingModulesMixin, CoverageTest):
@@ -443,13 +446,10 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
         assert squeezed[5] == "1 file skipped due to complete coverage."
 
     def test_report_skip_covered_no_data(self):
-        report = self.report_from_command("coverage report --skip-covered")
-
-        # No data to report.
-
-        assert self.line_count(report) == 1, report
-        squeezed = self.squeezed_lines(report)
-        assert squeezed[0] == "No data to report."
+        cov = coverage.Coverage()
+        cov.load()
+        with pytest.raises(NoDataError, match="No data to report."):
+            self.get_report(cov, skip_covered=True)
 
     def test_report_skip_empty(self):
         self.make_file("main.py", """
@@ -545,82 +545,71 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
         # We run a .py file, and when reporting, we can't parse it as Python.
         # We should get an error message in the report.
 
-        self.make_mycode()
-        self.run_command("coverage run mycode.py")
+        self.make_data_file(lines={"mycode.py": [1]})
         self.make_file("mycode.py", "This isn't python at all!")
-        report = self.report_from_command("coverage report mycode.py")
+        cov = coverage.Coverage()
+        cov.load()
+        msg = r"Couldn't parse '.*[/\\]mycode.py' as Python source: '.*' at line 1"
+        with pytest.raises(NotPython, match=msg):
+            self.get_report(cov, morfs=["mycode.py"])
 
-        # Couldn't parse '...' as Python source: 'invalid syntax' at line 1
-        # Name     Stmts   Miss  Cover
-        # ----------------------------
-        # No data to report.
+    def test_accented_directory(self):
+        # Make a file with a non-ascii character in the directory name.
+        self.make_file("\xe2/accented.py", "print('accented')")
+        self.make_data_file(lines={abs_file("\xe2/accented.py"): [1]})
+        report_expected = (
+            "Name            Stmts   Miss  Cover\n" +
+            "-----------------------------------\n" +
+            "\xe2/accented.py       1      0   100%\n" +
+            "-----------------------------------\n" +
+            "TOTAL               1      0   100%\n"
+        )
 
-        errmsg = self.squeezed_lines(report)[0]
-        # The actual file name varies run to run.
-        errmsg = re.sub(r"parse '.*mycode.py", "parse 'mycode.py", errmsg)
-        # The actual error message varies version to version
-        errmsg = re.sub(r": '.*' at", ": 'error' at", errmsg)
-        assert errmsg == "Couldn't parse 'mycode.py' as Python source: 'error' at line 1"
+        cov = coverage.Coverage()
+        cov.load()
+        output = self.get_report(cov, squeeze=False)
+        assert output == report_expected
 
     @pytest.mark.skipif(env.JYTHON, reason="Jython doesn't like accented file names")
     def test_accenteddotpy_not_python(self):
         # We run a .py file with a non-ascii name, and when reporting, we can't
         # parse it as Python.  We should get an error message in the report.
 
-        self.make_file("accented\xe2.py", "print('accented')")
-        self.run_command("coverage run accented\xe2.py")
+        self.make_data_file(lines={"accented\xe2.py": [1]})
         self.make_file("accented\xe2.py", "This isn't python at all!")
-        report = self.report_from_command("coverage report accented\xe2.py")
-
-        # Couldn't parse '...' as Python source: 'invalid syntax' at line 1
-        # Name     Stmts   Miss  Cover
-        # ----------------------------
-        # No data to report.
-
-        errmsg = self.squeezed_lines(report)[0]
-        # The actual file name varies run to run.
-        errmsg = re.sub(r"parse '.*(accented.*?\.py)", r"parse '\1", errmsg)
-        # The actual error message varies version to version
-        errmsg = re.sub(r": '.*' at", ": 'error' at", errmsg)
-        expected = "Couldn't parse 'accented\xe2.py' as Python source: 'error' at line 1"
-        assert expected == errmsg
+        cov = coverage.Coverage()
+        cov.load()
+        msg = r"Couldn't parse '.*[/\\]accented\xe2.py' as Python source: '.*' at line 1"
+        with pytest.raises(NotPython, match=msg):
+            self.get_report(cov, morfs=["accented\xe2.py"])
 
     def test_dotpy_not_python_ignored(self):
         # We run a .py file, and when reporting, we can't parse it as Python,
         # but we've said to ignore errors, so there's no error reported,
         # though we still get a warning.
-        self.make_mycode()
-        self.run_command("coverage run mycode.py")
         self.make_file("mycode.py", "This isn't python at all!")
-        report = self.report_from_command("coverage report -i mycode.py")
-
-        # CoverageWarning: Couldn't parse Python file blah_blah/mycode.py (couldnt-parse)
-        #    (source line)
-        # Name     Stmts   Miss  Cover
-        # ----------------------------
-        # No data to report.
-
-        assert self.line_count(report) == 5
-        assert 'No data to report.' in report
-        assert '(couldnt-parse)' in report
+        self.make_data_file(lines={"mycode.py": [1]})
+        cov = coverage.Coverage()
+        cov.load()
+        with pytest.raises(NoDataError, match="No data to report."):
+            with pytest.warns(Warning) as warns:
+                self.get_report(cov, morfs=["mycode.py"], ignore_errors=True)
+        assert_coverage_warnings(
+            warns,
+            re.compile(r"Couldn't parse Python file '.*[/\\]mycode.py' \(couldnt-parse\)"),
+            )
 
     def test_dothtml_not_python(self):
         # We run a .html file, and when reporting, we can't parse it as
         # Python.  Since it wasn't .py, no error is reported.
 
-        # Run an "html" file
-        self.make_file("mycode.html", "a = 1")
-        self.run_command("coverage run mycode.html")
-        # Before reporting, change it to be an HTML file.
+        # Pretend to run an html file.
         self.make_file("mycode.html", "<h1>This isn't python at all!</h1>")
-        report = self.report_from_command("coverage report mycode.html")
-
-        # Name     Stmts   Miss  Cover
-        # ----------------------------
-        # No data to report.
-
-        assert self.line_count(report) == 3
-        assert 'No data to report.' in report
+        self.make_data_file(lines={"mycode.html": [1]})
+        cov = coverage.Coverage()
+        cov.load()
+        with pytest.raises(NoDataError, match="No data to report."):
+            self.get_report(cov, morfs=["mycode.html"])
 
     def test_report_no_extension(self):
         self.make_file("xxx", """\
@@ -631,11 +620,12 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
             if not b:
                 c = 6
             d = 7
-            print("xxx: %r %r %r %r" % (a, b, c, d))
+            print(f"xxx: {a} {b} {c} {d}")
             """)
-        out = self.run_command("coverage run --source=. xxx")
-        assert out == "xxx: 3 4 0 7\n"
-        report = self.report_from_command("coverage report")
+        self.make_data_file(lines={abs_file("xxx"): [2, 3, 4, 5, 7, 8]})
+        cov = coverage.Coverage()
+        cov.load()
+        report = self.get_report(cov)
         assert self.last_line_squeezed(report) == "TOTAL 7 1 86%"
 
     def test_report_with_chdir(self):
@@ -789,17 +779,17 @@ class ReportingReturnValueTest(CoverageTest):
     def test_report(self):
         cov = self.run_coverage()
         val = cov.report(include="*/doit.py")
-        assert round(abs(val-85.7), 1) == 0
+        assert math.isclose(val, 6 / 7 * 100)
 
     def test_html(self):
         cov = self.run_coverage()
         val = cov.html_report(include="*/doit.py")
-        assert round(abs(val-85.7), 1) == 0
+        assert math.isclose(val, 6 / 7 * 100)
 
     def test_xml(self):
         cov = self.run_coverage()
         val = cov.xml_report(include="*/doit.py")
-        assert round(abs(val-85.7), 1) == 0
+        assert math.isclose(val, 6 / 7 * 100)
 
 
 class SummaryReporterConfigurationTest(CoverageTest):
@@ -921,5 +911,5 @@ class SummaryReporterConfigurationTest(CoverageTest):
     def test_sort_report_by_invalid_option(self):
         # Sort the text report by a nonsense column.
         msg = "Invalid sorting option: 'Xyzzy'"
-        with pytest.raises(CoverageException, match=msg):
+        with pytest.raises(ConfigError, match=msg):
             self.get_summary_text(('report:sort', 'Xyzzy'))
